@@ -2,17 +2,17 @@ import os
 
 from .data_model.study import Study
 import compute
+from compute.predict import PredictorConfigManager, PredictorDataSetManager
 import data_model
-from .external import make_study_datapackage
+from .datapackage import make_study_datapackage
 import visualize
 
-__version__ = '0.1.1'
-
-try:
-    get_ipython().magic(u'matplotlib inline')
-except:
-    pass
+__version__ = '0.2.2'
 
+_neural_diff_chr22 = 'http://sauron.ucsd.edu/flotilla_projects/' \
+                     'neural_diff_chr22/datapackage.json'
+_shalek2013 = 'https://raw.githubusercontent.com/YeoLab/shalek2013/master/' \
+              'datapackage.json'
 
 def embark(study_name, load_species_data=True):
     """
@@ -23,12 +23,10 @@ def embark(study_name, load_species_data=True):
     data_package_url : str
         A URL to a datapackage.json file
 
-
     Returns
     -------
     study : flotilla.Study
         A biological study created from the data package specified
-
     """
     try:
         try:
