@@ -1,3 +1,8 @@
+"""
+Convenience functions for obtaining reasonable plotting colors
+"""
+
+
 import matplotlib as mpl
 import seaborn as sns
 import brewer2mpl
@@ -5,7 +10,10 @@ import brewer2mpl
 sns.set_palette('deep')
 deep = sns.color_palette('deep')
 
-set1 = map(mpl.colors.rgb2hex, brewer2mpl.get_map('Set1', 'qualitative', 9).mpl_colors)
+dark2 = map(mpl.colors.rgb2hex,
+            brewer2mpl.get_map('Dark2', 'qualitative', 8).mpl_colors)
+set1 = map(mpl.colors.rgb2hex,
+           brewer2mpl.get_map('Set1', 'qualitative', 9).mpl_colors)
 red = set1[0]
 blue = set1[1]
 green = set1[2]
