@@ -2,7 +2,6 @@
 Named `ipython_interact.py` rather than just `interact.py` to differentiate
 between IPython interactive visualizations vs D3 interactive visualizations.
 """
-import itertools
 import sys
 import warnings
 
@@ -11,12 +10,12 @@ import matplotlib.pyplot as plt
 
 
 
-# from ..compute.predict import default_classifier
+# from ..compute.predict import CLASSIFIER
+from flotilla.util import link_to_list
 from ..visualize.color import red
 from .network import NetworkerViz
 from .color import str_to_color
 from ..util import natural_sort
-from ..external import link_to_list
 
 default_classifier = 'ExtraTreesClassifier'
 default_regressor = 'ExtraTreesRegressor'
@@ -92,17 +91,18 @@ class Interactive(object):
                         featurewise=False,
                         x_pc=(1, 10), y_pc=(1, 10),
                         show_point_labels=False,
-                        list_link = '',
-                        savefile='data/last.pca.pdf'):
+                        list_link='', plot_violins=False,
+                        savefile=''):
 
         def do_interact(data_type='expression',
                         sample_subset=self.default_sample_subsets,
-                        feature_subset=self.default_feature_subset,
+                        feature_subset=self.default_feature_subsets,
                         featurewise=False,
                         list_link='',
                         x_pc=1, y_pc=2,
+                        plot_violins=True,
                         show_point_labels=False,
-                        savefile='data/last.pca.pdf'):
+                        savefile=''):
 
             for k, v in locals().iteritems():
                 if k == 'self':
@@ -130,22 +130,21 @@ class Interactive(object):
                                 featurewise=featurewise,
                                 x_pc=x_pc, y_pc=y_pc,
                                 show_point_labels=show_point_labels,
-                                feature_subset=feature_subset)
+                                feature_subset=feature_subset,
+                                plot_violins=plot_violins)
             if savefile != '':
                 # Make the directory if it's not already there
                 self.maybe_make_directory(savefile)
                 # f = plt.gcf()
                 pca.reduced_fig.savefig(savefile)
+
+                # add "violins" after the provided filename, but before the
+                # extension
                 violins_file = "_".join([".".join(savefile.split('.')[:-1]),
                                          'violins']) + "." + \
-                                         savefile.split('.')[-1]
-                pca.violins_fig.savefig(violins_file)
-
-            feature_subsets = list(
-                set(itertools.chain(*self.default_feature_subsets
-                                    .values())))
-
-        # self.plot_study_sample_legend()
+                               savefile.split('.')[-1]
+                if plot_violins:
+                    pca.violins_fig.savefig(violins_file)
 
         if feature_subsets is None:
             feature_subsets = Interactive.get_feature_subsets(self, data_types)
@@ -154,14 +153,14 @@ class Interactive(object):
             sample_subsets = self.default_sample_subsets
 
         return interact(do_interact,
-                 data_type=data_types,
-                 sample_subset=sample_subsets,
-                 feature_subset=feature_subsets + ['custom'],
-                 featurewise=featurewise,
-                 x_pc=x_pc, y_pc=y_pc,
-                 show_point_labels=show_point_labels,
-                 list_link=list_link,
-                 savefile=savefile)
+                        data_type=data_types,
+                        sample_subset=sample_subsets,
+                        feature_subset=feature_subsets + ['custom'],
+                        featurewise=featurewise,
+                        x_pc=x_pc, y_pc=y_pc,
+                        show_point_labels=show_point_labels,
+                        list_link=list_link, plot_violins=plot_violins,
+                        savefile=savefile)
 
     @staticmethod
     def interactive_graph(self, data_types=('expression', 'splicing'),
@@ -176,7 +175,7 @@ class Interactive(object):
                           weight_fun=None,
                           use_pc_1=True, use_pc_2=True, use_pc_3=True,
                           use_pc_4=True,
-                          savefile='data/last.graph.pdf'):
+                          savefile=''):
 
         from IPython.html.widgets import interact
 
@@ -192,7 +191,7 @@ class Interactive(object):
                         cov_std_cut=1.8, n_pcs=5,
                         feature_of_interest="RBFOX2",
                         draw_labels=False,
-                        savefile='data/last.graph.pdf'):
+                        savefile=''):
 
             for k, v in locals().iteritems():
                 if k == 'self':
@@ -230,22 +229,22 @@ class Interactive(object):
             weight_fun = NetworkerViz.weight_funs
 
         # if not featurewise:
-        #     self.plot_study_sample_legend()
+        # self.plot_study_sample_legend()
 
         return interact(do_interact,
-                 data_type=data_types,
-                 sample_subset=sample_subsets,
-                 feature_subset=feature_subsets,
-                 featurewise=featurewise,
-                 cov_std_cut=cov_std_cut,
-                 degree_cut=degree_cut,
-                 n_pcs=n_pcs,
-                 draw_labels=draw_labels,
-                 weight_fun=weight_fun,
-                 feature_of_interest=feature_of_interest,
-                 use_pc_1=use_pc_1, use_pc_2=use_pc_2,
-                 use_pc_3=use_pc_3, use_pc_4=use_pc_4,
-                 savefile=savefile)
+                        data_type=data_types,
+                        sample_subset=sample_subsets,
+                        feature_subset=feature_subsets,
+                        featurewise=featurewise,
+                        cov_std_cut=cov_std_cut,
+                        degree_cut=degree_cut,
+                        n_pcs=n_pcs,
+                        draw_labels=draw_labels,
+                        weight_fun=weight_fun,
+                        feature_of_interest=feature_of_interest,
+                        use_pc_1=use_pc_1, use_pc_2=use_pc_2,
+                        use_pc_3=use_pc_3, use_pc_4=use_pc_4,
+                        savefile=savefile)
 
     @staticmethod
     def interactive_classifier(self, data_types=('expression', 'splicing'),
@@ -255,7 +254,7 @@ class Interactive(object):
                                predictor_types=None,
                                score_coefficient=(0.1, 20),
                                draw_labels=False,
-                               savefile='data/last.clf.pdf'):
+                               savefile=''):
 
         def do_interact(data_type,
                         sample_subset,
@@ -263,7 +262,7 @@ class Interactive(object):
                         predictor_type=default_classifier,
                         categorical_variable='outlier',
                         score_coefficient=2,
-                        savefile='data/last.clf.pdf'):
+                        savefile=''):
 
             for k, v in locals().iteritems():
                 if k == 'self':
@@ -298,14 +297,14 @@ class Interactive(object):
         # self.plot_study_sample_legend()
 
         return interact(do_interact,
-                 data_type=data_types,
-                 sample_subset=sample_subsets,
-                 feature_subset=feature_subsets,
-                 categorical_variable=categorical_variables,
-                 score_coefficient=score_coefficient,
-                 draw_labels=draw_labels,
-                 predictor_type=predictor_types,
-                 savefile=savefile)
+                        data_type=data_types,
+                        sample_subset=sample_subsets,
+                        feature_subset=feature_subsets,
+                        categorical_variable=categorical_variables,
+                        score_coefficient=score_coefficient,
+                        draw_labels=draw_labels,
+                        predictor_type=predictor_types,
+                        savefile=savefile)
 
     @staticmethod
     def interactive_localZ(self):
@@ -347,10 +346,10 @@ class Interactive(object):
                              "<this_object>.localZ_result_")
 
         return interact(do_interact,
-                 data_type=('expression', 'splicing'),
-                 sample1='replaceme',
-                 sample2='replaceme',
-                 pCut='0.01')
+                        data_type=('expression', 'splicing'),
+                        sample1='replaceme',
+                        sample2='replaceme',
+                        pCut='0.01')
 
     @staticmethod
     def interactive_plot_modalities_lavalamps(self, sample_subsets=None,
@@ -417,11 +416,13 @@ class Interactive(object):
             bootstrapped_kws = {}
 
         return interact(do_interact,
-                 sample_subset=sample_subsets, feature_subset=feature_subsets,
-                 color=color, x_offset=x_offset,
-                 use_these_modalities=use_these_modalities,
-                 bootstrapped=bootstrapped, bootstrapped_kws=bootstrapped_kws,
-                 savefile=savefile)
+                        swample_subset=sample_subsets,
+                        feature_subset=feature_subsets,
+                        color=color, x_offset=x_offset,
+                        use_these_modalities=use_these_modalities,
+                        bootstrapped=bootstrapped,
+                        bootstrapped_kws=bootstrapped_kws,
+                        savefile=savefile)
 
     @staticmethod
     def interactive_lavalamp_pooled_inconsistent(
@@ -465,62 +466,164 @@ class Interactive(object):
             sample_subsets = self.default_sample_subsets
 
         return interact(do_interact,
-                 sample_subset=sample_subsets,
-                 feature_subset=feature_subsets,
-                 difference_threshold=difference_threshold,
-                 color=colors,
-                 savefile='')
+                        sample_subset=sample_subsets,
+                        feature_subset=feature_subsets,
+                        difference_threshold=difference_threshold,
+                        color=colors,
+                        savefile='')
 
-    # @staticmethod
-    # def interactive_clusteredheatmap(self):
-    #     def do_interact(data_type='expression',
-    #                     sample_subset=self.default_sample_subsets,
-    #                     feature_subset=self.default_feature_subset,
-    #                     metric='euclidean',
-    #                     linkage_method='median',
-    #                     list_link='',
-    #                     savefile='data/last.clusteredheatmap.pdf'):
-    #
-    #         for k, v in locals().iteritems():
-    #             if k == 'self':
-    #                 continue
-    #             sys.stdout.write('{} : {}\n'.format(k, v))
-    #
-    #         if feature_subset != "custom" and list_link != "":
-    #             raise ValueError(
-    #                 "set feature_subset to \"custom\" to use list_link")
-    #
-    #         if feature_subset == "custom" and list_link == "":
-    #             raise ValueError("use a custom list name please")
-    #
-    #         if feature_subset == 'custom':
-    #             feature_subset = list_link
-    #         elif feature_subset not in self.default_feature_subsets[data_type]:
-    #             warnings.warn("This feature_subset ('{}') is not available in "
-    #                           "this data type ('{}'). Falling back on all "
-    #                           "features.".format(feature_subset, data_type))
-    #
-    #         self.plot_clusteredheatmap(sample_subset=sample_subset,
-    #                                    feature_subset=feature_subset,
-    #                                    data_type=data_type,
-    #                                    metric=metric,
-    #                                    linkage_method=linkage_method)
-    #         plt.tight_layout()
-    #         if savefile != '':
-    #             # Make the directory if it's not already there
-    #             self.maybe_make_directory(savefile)
-    #             f = plt.gcf()
-    #             f.savefig(savefile)
-    #
-    #     feature_subsets = Interactive.get_feature_subsets(self,
-    #                                                       ['expression',
-    #                                                        'splicing'])
-    #
-    #     linkage_method = ('single', 'median', 'centroid')
-    #     metric = ('euclidean', 'seuclidean')
-    #     interact(do_interact,
-    #              data_type=('expression', 'splicing'),
-    #              sample_subset=self.default_sample_subsets,
-    #              feature_subset=feature_subsets,
-    #              metric=metric,
-    #              linkage_method=linkage_method)
+    @staticmethod
+    def interactive_choose_outliers(self,
+                                    data_types=('expression', 'splicing'),
+                                    sample_subsets=None,
+                                    feature_subsets=None,
+                                    featurewise=False,
+                                    x_pc=(1, 3), y_pc=(1, 3),
+                                    show_point_labels=False,
+                                    kernel=['rbf','linear','poly','sigmoid'],
+                                    gamma=(0, 25),
+                                    nu=(0.1, 9.9),
+                                    ):
+
+        def do_interact(data_type='expression',
+                        sample_subset=self.default_sample_subset,
+                        feature_subset=self.default_feature_subset,
+                        x_pc=1, y_pc=2,
+                        show_point_labels=False,
+                        kernel='rbf',
+                        gamma=16,
+                        nu=.2,
+                        ):
+            print "transforming input gamma by 2^-(input): %f" % gamma
+            gamma = 2 ** -float(gamma)
+            print "transforming input nu by input/10: %f" % nu
+            nu = float(nu) / 10
+            kernel = str(kernel)
+            for k, v in locals().iteritems():
+                if k == 'self':
+                    continue
+                sys.stdout.write('{} : {}\n'.format(k, v))
+
+            else:
+                renamer = lambda x: x
+                data_model = None
+
+            if feature_subset not in self.default_feature_subsets[data_type]:
+                warnings.warn("This feature_subset ('{}') is not available in "
+                              "this data type ('{}'). Falling back on all "
+                              "features.".format(feature_subset, data_type))
+
+            reducer, outlier_detector = self.detect_outliers(data_type=data_type,
+                                                             sample_subset=sample_subset,
+                                                             feature_subset=feature_subset,
+                                                             reducer=None,
+                                                             reducer_kwargs=None,
+                                                             outlier_detection_method=None,
+                                                             outlier_detection_method_kwargs={'gamma': gamma,
+                                                                                              'nu': nu,
+                                                                                              'kernel': kernel})
+            if data_type == "expression":
+                obj = self.expression
+            if data_type == "splicing":
+                obj = self.splicing
+
+            obj.plot_outliers(reducer, outlier_detector,
+                              feature_renamer=renamer,
+                              show_point_labels=show_point_labels,
+                              x_pc="pc_" + str(x_pc),
+                              y_pc="pc_" + str(y_pc))
+
+            print "total samples:", len(outlier_detector.outliers)
+            print "outlier samples:", outlier_detector.outliers.sum()
+            print "metadata column name:", outlier_detector.title
+
+        if feature_subsets is None:
+            feature_subsets = Interactive.get_feature_subsets(self, data_types)
+
+        if sample_subsets is None:
+            sample_subsets = self.default_sample_subsets
+
+        return interact(do_interact,
+                        data_type=data_types,
+                        sample_subset=sample_subsets,
+                        feature_subset=feature_subsets,
+                        x_pc=x_pc, y_pc=y_pc,
+                        show_point_labels=show_point_labels,
+                        kernel=kernel,
+                        nu=nu,
+                        gamma=gamma)
+
+    @staticmethod
+    def interactive_reset_outliers(self):
+        """User selects from columns that start with 'outlier_' to merge
+        multiple outlier classifications"""
+        outlier_columns = dict()
+
+        for column in self.metadata.data.columns:
+            if column.startswith("outlier_"):
+                outlier_columns[column] = False
+
+        def do_interact(**columns):
+            if len(columns.keys()) == 0:
+                print "You have not specified any 'outlier_' columns in study.metadata.data... \n" \
+                      "This function will do nothing to your data."
+            else:
+                self.metadata.set_outliers_by_merging_columns(
+                    [k for (k, v) in columns.items() if v])
+
+        interact(do_interact, **outlier_columns)
+
+
+        # @staticmethod
+        # def interactive_clusteredheatmap(self):
+        # def do_interact(data_type='expression',
+        #                     sample_subset=self.default_sample_subsets,
+        #                     feature_subset=self.default_feature_subset,
+        #                     metric='euclidean',
+        #                     linkage_method='median',
+        #                     list_link='',
+        #                     savefile='data/last.clusteredheatmap.pdf'):
+        #
+        #         for k, v in locals().iteritems():
+        #             if k == 'self':
+        #                 continue
+        #             sys.stdout.write('{} : {}\n'.format(k, v))
+        #
+        #         if feature_subset != "custom" and list_link != "":
+        #             raise ValueError(
+        #                 "set feature_subset to \"custom\" to use list_link")
+        #
+        #         if feature_subset == "custom" and list_link == "":
+        #             raise ValueError("use a custom list name please")
+        #
+        #         if feature_subset == 'custom':
+        #             feature_subset = list_link
+        #         elif feature_subset not in self.default_feature_subsets[data_type]:
+        #             warnings.warn("This feature_subset ('{}') is not available in "
+        #                           "this data type ('{}'). Falling back on all "
+        #                           "features.".format(feature_subset, data_type))
+        #
+        #         self.plot_clusteredheatmap(sample_subset=sample_subset,
+        #                                    feature_subset=feature_subset,
+        #                                    data_type=data_type,
+        #                                    metric=metric,
+        #                                    linkage_method=linkage_method)
+        #         plt.tight_layout()
+        #         if savefile != '':
+        #             # Make the directory if it's not already there
+        #             self.maybe_make_directory(savefile)
+        #             f = plt.gcf()
+        #             f.savefig(savefile)
+        #
+        #     feature_subsets = Interactive.get_feature_subsets(self,
+        #                                                       ['expression',
+        #                                                        'splicing'])
+        #
+        #     linkage_method = ('single', 'median', 'centroid')
+        #     metric = ('euclidean', 'seuclidean')
+        #     interact(do_interact,
+        #              data_type=('expression', 'splicing'),
+        #              sample_subset=self.default_sample_subsets,
+        #              feature_subset=feature_subsets,
+        #              metric=metric,
+        #              linkage_method=linkage_method)
